Package: rTensor
Type: Package
Title: Tools for Tensor Analysis and Decomposition
Version: 1.5.0
Authors@R: c(person(given = "James",
                    family = "Li",
                    role = "aut"),
             person(given = "Jacob",
                    family = "Bien",
                    role = "aut"),
             person(given = "Martin",
                    family = "Wells",
                    role = "aut"),
             person(given = "Koki",
                    family = "Tsuyuzaki",
                    role = c("cre", "ctb"),
                    email = "k.t.the-answer@hotmail.co.jp"))
Description: A set of tools for creation, manipulation, and modeling
    of tensors with arbitrary number of modes. A tensor in the context of data
    analysis is a multidimensional array. rTensor does this by providing a S4
    class 'Tensor' that wraps around the base 'array' class. rTensor
    provides common tensor operations as methods, including matrix unfolding,
    summing/averaging across modes, calculating the Frobenius norm, and taking
    the inner product between two tensors. Familiar array operations are
    overloaded, such as index subsetting via '[' and element-wise operations.
    rTensor also implements various tensor decomposition, including CP, GLRAM,
    MPCA, PVD, Tucker, INDSCAL, RESCAL, DEDICOM, PARAFAC2, and 2DLDA. For
    tensors with 3 modes, rTensor also implements
    transpose, t-product, and t-SVD, as defined in Kilmer et al. (2013). Some
    auxiliary functions include the Khatri-Rao product, Kronecker product, and
    the Hadamard product for a list of matrices.
License: MIT + file LICENSE
Encoding: UTF-8
Depends: R (>= 2.10.0)
Imports: methods
Suggests: testthat
Date: 2026-04-12
URL: https://github.com/rikenbit/rTensor
Config/testthat/edition: 2
RoxygenNote: 7.1.2
NeedsCompilation: no
Packaged: 2026-05-07 04:50:09 UTC; koki
Author: James Li [aut],
  Jacob Bien [aut],
  Martin Wells [aut],
  Koki Tsuyuzaki [cre, ctb]
Maintainer: Koki Tsuyuzaki <k.t.the-answer@hotmail.co.jp>
Repository: CRAN
Date/Publication: 2026-05-09 14:41:26 UTC
Built: R 4.6.0; ; 2026-05-09 17:35:37 UTC; unix
