# Generated by roxygen2: do not edit by hand

export(as.tensor)
export(cp)
export(cs_fold)
export(cs_unfold)
export(fnorm)
export(fold)
export(hadamard_list)
export(hosvd)
export(innerProd)
export(k_fold)
export(k_unfold)
export(khatri_rao)
export(khatri_rao_list)
export(kronecker_list)
export(matvec)
export(modeMean)
export(modeSum)
export(mpca)
export(plot_orl)
export(pvd)
export(rand_tensor)
export(rs_fold)
export(rs_unfold)
export(t_mult)
export(t_svd)
export(t_svd_reconstruct)
export(tperm)
export(ttl)
export(ttm)
export(tucker)
export(unfold)
export(unmatvec)
export(vec)
export(load_orl)
export(indscal)
export(rescal)
export(dedicom)
export(parafac2)
export(twodlda)

exportClasses(Tensor)

exportMethods("[")
exportMethods(dim)
exportMethods(head)
exportMethods(print)
exportMethods(show)
exportMethods(t)
exportMethods(tail)

importFrom(grDevices, grey)
importFrom(graphics, image)
importFrom(methods, callGeneric, is, new, show, validObject)
importFrom(stats, fft, rnorm)
importFrom(utils, data, setTxtProgressBar, txtProgressBar, download.file)