% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/rTensor_Class.R
\docType{methods}
\name{dim-methods}
\alias{dim-methods}
\alias{dim,Tensor-method}
\title{Mode Getter for Tensor}
\usage{
\S4method{dim}{Tensor}(x)
}
\arguments{
\item{x}{the Tensor instance}
}
\value{
an integer vector of the modes associated with \code{x}
}
\description{
Return the vector of modes from a tensor
}
\details{
\code{dim(x)}
}
\examples{
tnsr <- rand_tensor()
dim(tnsr)
}
